﻿using System;
using System.ComponentModel;
using System.Windows.Media;

namespace JPing
{
    /// <summary>
    /// JPing setting.
    /// </summary>
    public class Setting : INotifyPropertyChanged
    {
        #region Events

        /// <summary>
        /// Triggered when specified property value has changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Trigger the PropertyChanged event.
        /// </summary>
        /// <param name="name">Property name</param>
        protected void OnPropertyChanged(string name)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(name));
        }

        #endregion


        #region Properties

        /// <summary>
        /// Get or set the window position X.
        /// </summary>
        public double Left { get; set; }

        /// <summary>
        /// Get or set the window position Y.
        /// </summary>
        public double Top { get; set; }

        /// <summary>
        /// Ping target host.
        /// </summary>
        private string _target = "localhost";

        /// <summary>
        /// Get or set the ping target host.
        /// </summary>
        public string Target
        {
            get => _target;
            set
            {
                if (string.IsNullOrWhiteSpace(value))
                    value = "localhost";
                else
                    value = value.Trim();
                if (value == _target) return;
                _target = value;
                OnPropertyChanged("Target");
            }
        }

        /// <summary>
        /// Ping interval in milliseconds.
        /// </summary>
        private float _interval = 1;

        /// <summary>
        /// Get or set the ping interval in milliseconds.
        /// </summary>
        public float Interval
        {
            get => _interval;
            set
            {
                if (value < 0.1F) value = 0.1F;
                if (value == _interval) return;
                _interval = value;
                OnPropertyChanged("Interval");
            }
        }

        /// <summary>
        /// Average count of ping value.
        /// </summary>
        private int _average = 3;

        /// <summary>
        /// Get or set the average count of ping value.
        /// </summary>
        public int Average
        {
            get => _average;
            set
            {
                if (value < 1) value = 1;
                else if (value > 100) value = 100;
                if (value == _average) return;
                _average = value;
                OnPropertyChanged("Average");
            }
        }

        /// <summary>
        /// Threshold of low ping value.
        /// </summary>
        private int _lowerGate = 10;

        /// <summary>
        /// Get or set the threshold of low ping value.
        /// </summary>
        public int LowerGate
        {
            get => _lowerGate;
            set
            {
                if (value > _upperGate) value = _upperGate;
                else if (value < 0) value = 0;
                if (value == _lowerGate) return;
                _lowerGate = value;
                OnPropertyChanged("LowerGate");
            }
        }

        /// <summary>
        /// Get or set the color of low ping value.
        /// </summary>
        public Color LowerColor { get; set; } = Colors.LightGreen;

        /// <summary>
        /// Threshold of high ping value.
        /// </summary>
        private int _upperGate = 75;

        /// <summary>
        /// Get or set the threshold of high ping value.
        /// </summary>
        public int UpperGate
        {
            get => _upperGate;
            set
            {
                if (value < _lowerGate) value = _lowerGate;
                else if (value > 1000) value = 1000;
                if (value == _upperGate) return;
                _upperGate = value;
                OnPropertyChanged("UpperGate");
            }
        }

        /// <summary>
        /// Get or set the color of high ping value.
        /// </summary>
        public Color UpperColor { get; set; } = Colors.LightPink;

        /// <summary>
        /// Get or set that whether use an additional color for median ping value.
        /// </summary>
        public bool UseMiddleColor { get; set; } = true;

        /// <summary>
        /// Get or set the color of median ping value.
        /// </summary>
        public Color MiddleColor { get; set; } = Colors.Gold;

        /// <summary>
        /// Get or set that whether use an additional color for timeout ping value.
        /// </summary>
        public bool UseTimeoutColor { get; set; } = true;

        /// <summary>
        /// Get or set the color of timeout ping value.
        /// </summary>
        public Color TimeoutColor { get; set; } = Colors.Red;

        /// <summary>
        /// Whether display the ping value in text.
        /// </summary>
        private bool _displayPingValue = true;

        /// <summary>
        /// Get or set that whether display the ping value in text.
        /// </summary>
        public bool DisplayPingValue
        {
            get => _displayPingValue;
            set
            {
                if (value == _displayPingValue) return;
                _displayPingValue = value;
                OnPropertyChanged("DisplayPingValue");
            }
        }

        /// <summary>
        /// Color of text.
        /// </summary>
        private Color _textColor = Colors.Black;

        /// <summary>
        /// Get or set the color of text.
        /// </summary>
        public Color TextColor
        {
            get => _textColor;
            set
            {
                if (value == _textColor) return;
                _textColor = value;
                OnPropertyChanged("TextColor");
            }
        }

        #endregion


        #region Color Methods

        /// <summary>
        /// Get the blended color of specified ping value.
        /// </summary>
        /// <param name="value">Ping value</param>
        /// <returns>Blended color</returns>
        public Color GetColor(double value)
        {
            double f = (value - _lowerGate) / (_upperGate - _lowerGate);
            return UseMiddleColor ? CalculateColorWithMiddleColor(f) : MixColor(UpperColor, LowerColor, f);
        }

        /// <summary>
        /// Calculate the blended color when using medium color.
        /// </summary>
        /// <param name="ratio">Ping value ratio between low and high threshold</param>
        /// <returns>Blended color</returns>
        private Color CalculateColorWithMiddleColor(double ratio)
        {
            if (ratio > 0.5)
                return MixColor(UpperColor, MiddleColor, (ratio - 0.5) * 2);
            else if (ratio < 0.5)
                return MixColor(MiddleColor, LowerColor, ratio * 2);
            else
                return MiddleColor;
        }

        /// <summary>
        /// Mixing two colors with a specified ratio.
        /// The range of ratio is between 0 and 1.
        /// </summary>
        /// <param name="c1">Color 1</param>
        /// <param name="c2">Color 2</param>
        /// <param name="ratio">The ratio of color 1, the range is 0 ~ 1</param>
        /// <returns>Mixed color</returns>
        private Color MixColor(Color c1, Color c2, double ratio)
        {
            if (ratio >= 1) return c1;
            else if (ratio <= 0) return c2;
            return Color.FromArgb(255, ByteMix(c1.R, c2.R, ratio), ByteMix(c1.G, c2.G, ratio), ByteMix(c1.B, c2.B, ratio));
        }

        /// <summary>
        /// Mixing 2 channels of color with a ratio.
        /// </summary>
        /// <param name="a">Channel 1</param>
        /// <param name="b">Channel 2</param>
        /// <param name="ratio">The ratio of channel 1</param>
        /// <returns>Mixed channel value</returns>
        private byte ByteMix(byte a, byte b, double ratio)
        {
            if (ratio < 0 || ratio > 1)
                throw new ArgumentOutOfRangeException("ratio");
            return (byte)(a * ratio + b * (1 - ratio));
        }

        #endregion
    }
}
